@@ -41,7 +41,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/*
+/**
  * SECTION: metadataexif
  * @short_description: This module provides functions to extract tags from
  * EXIF metadata chunks and create EXIF chunks from metadata tags.
@@ -216,6 +216,9 @@ static MapIntStr mappedTags[] = {
   {EXIF_TAG_DATE_TIME_ORIGINAL,          /*ASCII,*/     EXIF_IFD_EXIF,
    GST_TAG_DATE_TIME_ORIGINAL            /*G_TYPE_STRING*/},
 
+  {EXIF_TAG_IMAGE_DESCRIPTION,           /*ASCII,*/     EXIF_IFD_0,
+   GST_TAG_DESCRIPTION                   /*G_TYPE_STRING*/},
+
   {EXIF_TAG_MAKE,                        /*ASCII,*/     EXIF_IFD_0,
    GST_TAG_DEVICE_MAKE                   /*G_TYPE_STRING*/},
 
@@ -754,11 +757,12 @@ metadataparse_handle_unit_tags (ExifEntry * entry, MEUserData * meudata,
 
       break;
     case EXIF_TAG_GPS_ALTITUDE_REF:
-      meudata->altitude_ref = entry->data[0];
-
+    {
       const GValue *value = gst_tag_list_get_value_index (meudata->taglist,
           GST_TAG_GPS_ALTITUDE, 0);
 
+      meudata->altitude_ref = entry->data[0];
+
       if (value) {
         gint n, d;
 
@@ -771,7 +775,7 @@ metadataparse_handle_unit_tags (ExifEntry * entry, MEUserData * meudata,
           }
         }
       }
-
+    }
       break;
     case EXIF_TAG_GPS_LATITUDE_REF:
     {
