@@ -17,48 +17,38 @@
 
 /**
  * SECTION:element-mythtvsrc
- * @short_description: stream from remore mythtv server
+ * @see_also: nuvdemux
  *
- * <refsect2>
- * <para>
  * MythTVSrc allows to access a remote MythTV backend streaming Video/Audio server,
  * and to render audio and video content through a TCP/IP connection to a specific
  * port on this server, and based on a known MythTV protocol that is based on 
  * some message passing, such as REQUEST_BLOCK on a specified number of bytes, to get
  * some chunk of remote file data.
  * You should pass the information aboute the remote MythTV backend server 
- * through the <link linkend="GstMythTVSrc--location">location</link> property.
- * </para>
+ * through the #GstMythtvSrc:location property.
+ * 
+ * <refsect2>
  * <title>Examples</title>
  * <para>
  * If you want to get the LiveTV content (set channel, TV tuner, RemoteEncoder, 
- * Recorder),
- * put the following URI:
- * 
- * <programlisting> 
+ * Recorder), use the following URI:
+ * <programlisting>
  *  myth://xxx.xxx.xxx.xxx:6543/livetv?channel=BBC
  * </programlisting>
  *
- * This URI will say to the gmyth library to configure the Recorder instance (used to
- * change the channel, start the TV multimedia content transmition, etc.), using
+ * This URI will configure the Recorder instance (used to change the channel,
+ * start the TV multimedia content transmition, etc.), using
  * the IP address (xxx.xxx.xxx.xxx) and port number (6543) of the MythTV backend 
  * server, and setting the channel name to "BBC". 
  * 
  * To get a already recorded the MythTV NUV file, put the following URI:
- * 
  * <programlisting>
  *  myth://xxx.xxx.xxx.xxx:6543/filename.nuv
  * </programlisting>
- *
- * This URI will say to the gmyth library to configure the Recorder instance (used to
- * change the channel, start the TV multimedia content transmition, etc.), using
- * the IP address (xxx.xxx.xxx.xxx) and port number (6543) of the MythTV backend 
- * server, and setting the channel name to "BBC".
  * 
  * Another possible way to use the LiveTV content, and just in the case you want to 
  * use the mysql database, put the location URI in the following format:
- * 
- * <programlisting> 
+ * <programlisting>
  *  myth://mythtv:mythtv@xxx.xxx.xxx.xxx:6543/?mythconverg&channel=9
  * </programlisting>
  * 
@@ -132,7 +122,7 @@ static gboolean gst_mythtv_src_stop (GstBaseSrc * bsrc);
 static gboolean gst_mythtv_src_get_size (GstBaseSrc * bsrc, guint64 * size);
 static gboolean gst_mythtv_src_is_seekable (GstBaseSrc * push_src);
 
-static gboolean gst_mythtv_src_do_seek (GstBaseSrc * base,
+static GstFlowReturn gst_mythtv_src_do_seek (GstBaseSrc * base,
     GstSegment * segment);
 
 static GstStateChangeReturn
@@ -146,9 +136,10 @@ static void gst_mythtv_src_get_property (GObject * object,
 static void gst_mythtv_src_uri_handler_init (gpointer g_iface,
     gpointer iface_data);
 
+#if 0
 static gboolean gst_mythtv_src_handle_query (GstPad * pad, GstQuery * query);
-
 static gboolean gst_mythtv_src_handle_event (GstPad * pad, GstEvent * event);
+#endif
 
 static GMythFileReadResult do_read_request_response (GstMythtvSrc * src,
     guint size, GByteArray * data_ptr);
@@ -241,12 +232,13 @@ gst_mythtv_src_class_init (GstMythtvSrcClass * klass)
       g_param_spec_string ("mythtv-channel", "mythtv-channel",
           "Change MythTV channel number", "", G_PARAM_READWRITE));
 
-  gstbasesrc_class->start = gst_mythtv_src_start;
-  gstbasesrc_class->stop = gst_mythtv_src_stop;
-  gstbasesrc_class->get_size = gst_mythtv_src_get_size;
-  gstbasesrc_class->is_seekable = gst_mythtv_src_is_seekable;
-  gstbasesrc_class->do_seek = gst_mythtv_src_do_seek;
-  gstpushsrc_class->create = gst_mythtv_src_create;
+  gstbasesrc_class->start = GST_DEBUG_FUNCPTR (gst_mythtv_src_start);
+  gstbasesrc_class->stop = GST_DEBUG_FUNCPTR (gst_mythtv_src_stop);
+  gstbasesrc_class->get_size = GST_DEBUG_FUNCPTR (gst_mythtv_src_get_size);
+  gstbasesrc_class->is_seekable =
+      GST_DEBUG_FUNCPTR (gst_mythtv_src_is_seekable);
+  gstbasesrc_class->do_seek = GST_DEBUG_FUNCPTR (gst_mythtv_src_do_seek);
+  gstpushsrc_class->create = GST_DEBUG_FUNCPTR (gst_mythtv_src_create);
 
   GST_DEBUG_CATEGORY_INIT (mythtvsrc_debug, "mythtvsrc", 0,
       "MythTV Client Source");
@@ -274,10 +266,14 @@ gst_mythtv_src_init (GstMythtvSrc * this, GstMythtvSrcClass * g_class)
   this->wait_to_transfer = 0;
   this->spawn_livetv = NULL;
   gst_base_src_set_format (GST_BASE_SRC (this), GST_FORMAT_BYTES);
+#if 0
   gst_pad_set_event_function (GST_BASE_SRC_PAD (GST_BASE_SRC (this)),
       gst_mythtv_src_handle_event);
+#endif
+#if 0
   gst_pad_set_query_function (GST_BASE_SRC_PAD (GST_BASE_SRC (this)),
       gst_mythtv_src_handle_query);
+#endif
 
 }
 
@@ -286,6 +282,7 @@ gst_mythtv_src_clear (GstMythtvSrc * mythtv_src)
 {
   mythtv_src->unique_setup = FALSE;
 
+#if 0
   if (mythtv_src->spawn_livetv) {
     g_object_unref (mythtv_src->spawn_livetv);
     mythtv_src->spawn_livetv = NULL;
@@ -300,6 +297,7 @@ gst_mythtv_src_clear (GstMythtvSrc * mythtv_src)
     g_object_unref (mythtv_src->backend_info);
     mythtv_src->backend_info = NULL;
   }
+#endif
 }
 
 static void
@@ -380,6 +378,7 @@ eos:
   src->eos = TRUE;
 
 done:
+  GST_LOG_OBJECT (src, "Finished read: result %d", result);
   return result;
 }
 
@@ -402,7 +401,6 @@ gst_mythtv_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
   if (result == GMYTH_FILE_READ_ERROR)
     goto read_error;
 
-
   *outbuf = gst_buffer_new ();
   GST_BUFFER_SIZE (*outbuf) = buffer->len;
   GST_BUFFER_MALLOCDATA (*outbuf) = buffer->data;
@@ -429,6 +427,7 @@ gst_mythtv_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
   if (src->eos || (!src->live_tv && (src->bytes_read >= src->content_size)))
     ret = GST_FLOW_UNEXPECTED;
 
+  GST_LOG_OBJECT (src, "Create finished: %d", ret);
   return ret;
 
 read_error:
@@ -522,6 +521,7 @@ gst_mythtv_src_start (GstBaseSrc * bsrc)
   GString *chain_id_local = NULL;
   GMythURI *gmyth_uri = NULL;
   gboolean ret = TRUE;
+  GstBaseSrc *basesrc;
   GstMessage *msg;
 
   if (src->unique_setup == FALSE) {
@@ -535,8 +535,8 @@ gst_mythtv_src_start (GstBaseSrc * bsrc)
   src->live_tv = gmyth_uri_is_livetv (gmyth_uri);
 
   if (src->live_tv) {
-    src->spawn_livetv = gmyth_livetv_new (src->backend_info);
     gchar *ch = gmyth_uri_get_channel_name (gmyth_uri);
+    src->spawn_livetv = gmyth_livetv_new (src->backend_info);
 
     if (ch != NULL)
       src->channel_name = ch;
@@ -629,9 +629,17 @@ gst_mythtv_src_start (GstBaseSrc * bsrc)
 
   src->do_start = FALSE;
 
+  basesrc = GST_BASE_SRC_CAST (src);
+  gst_segment_set_duration (&basesrc->segment, GST_FORMAT_BYTES,
+      src->content_size);
+  gst_element_post_message (GST_ELEMENT (src),
+      gst_message_new_duration (GST_OBJECT (src), GST_FORMAT_BYTES,
+          src->content_size));
+#if 0
   gst_pad_push_event (GST_BASE_SRC_PAD (GST_BASE_SRC (src)),
       gst_event_new_new_segment (TRUE, 1.0,
           GST_FORMAT_BYTES, 0, src->content_size, 0));
+#endif
 done:
   if (gmyth_uri != NULL) {
     g_object_unref (gmyth_uri);
@@ -724,6 +732,7 @@ gst_mythtv_src_stop (GstBaseSrc * bsrc)
   return TRUE;
 }
 
+#if 0
 static gboolean
 gst_mythtv_src_handle_event (GstPad * pad, GstEvent * event)
 {
@@ -751,13 +760,14 @@ gst_mythtv_src_handle_event (GstPad * pad, GstEvent * event)
   GST_DEBUG_OBJECT (src, "HANDLE EVENT %d", ret);
   return ret;
 }
-
+#endif
 static gboolean
 gst_mythtv_src_is_seekable (GstBaseSrc * push_src)
 {
   return TRUE;
 }
 
+#if 0
 static gboolean
 gst_mythtv_src_handle_query (GstPad * pad, GstQuery * query)
 {
@@ -798,7 +808,7 @@ gst_mythtv_src_handle_query (GstPad * pad, GstQuery * query)
 
   return res;
 }
-
+#endif
 static GstStateChangeReturn
 gst_mythtv_src_change_state (GstElement * element, GstStateChange transition)
 {
