@@ -17,6 +17,18 @@
  *
  */
 
+#define FLOAT_SAMPLES 1
+#include <soundtouch/SoundTouch.h>
+/* FIXME: workaround for SoundTouch.h of version 1.3.1 defining those
+ * variables while it shouldn't. */
+#undef VERSION
+#undef PACKAGE_VERSION
+#undef PACKAGE_TARNAME
+#undef PACKAGE_STRING
+#undef PACKAGE_NAME
+#undef PACKAGE_BUGREPORT
+#undef PACKAGE
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -26,16 +38,6 @@
 #include "gstpitch.hh"
 #include <math.h>
 
-#define FLOAT_SAMPLES
-#include <soundtouch/SoundTouch.h>
-
-/* wtf ?
-#ifdef G_PARAM_READWRITE
-#  undef G_PARAM_READWRITE
-#endif
-#define G_PARAM_READWRITE ((GParamFlags)(G_PARAM_READABLE | G_PARAM_WRITABLE))
-*/
-
 GST_DEBUG_CATEGORY_STATIC (pitch_debug);
 #define GST_CAT_DEFAULT pitch_debug
 
@@ -44,36 +46,26 @@ struct _GstPitchPrivate
 {
   gfloat stream_time_ratio;
 
+  GstEvent *pending_segment;
+
     soundtouch::SoundTouch * st;
 };
 
-static GstElementDetails gst_pitch_details =
-GST_ELEMENT_DETAILS ("Pitch controller",
-    "Filter/Converter/Audio",
-    "Control the pitch of an audio stream",
-    "Wouter Paesen <wouter@kangaroot.net>");
-
-enum
-{
-  LAST_SIGNAL
-};
-
 enum
 {
   ARG_0,
   ARG_RATE,
   ARG_TEMPO,
-  ARG_PITCH,
+  ARG_PITCH
 };
 
 #define SUPPORTED_CAPS \
 GST_STATIC_CAPS( \
   "audio/x-raw-float, " \
-    "rate = (int) [ 8000, 48000 ], " \
+    "rate = (int) [ 8000, MAX ], " \
     "channels = (int) [ 1, 2 ], " \
     "endianness = (int) BYTE_ORDER, " \
-    "width = (int) 32, " \
-    "buffer-frames = (int) [ 0, MAX ]" \
+    "width = (int) 32" \
 )
 
 static GstStaticPadTemplate gst_pitch_sink_template =
@@ -117,7 +109,9 @@ gst_pitch_base_init (gpointer g_class)
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&gst_pitch_sink_template));
 
-  gst_element_class_set_details (gstelement_class, &gst_pitch_details);
+  gst_element_class_set_details_simple (gstelement_class, "Pitch controller",
+      "Filter/Converter/Audio", "Control the pitch of an audio stream",
+      "Wouter Paesen <wouter@blue-gate.be>");
 }
 
 static void
@@ -129,6 +123,9 @@ gst_pitch_class_init (GstPitchClass * klass)
   gobject_class = G_OBJECT_CLASS (klass);
   element_class = GST_ELEMENT_CLASS (klass);
 
+  GST_DEBUG_CATEGORY_INIT (pitch_debug, "pitch", 0,
+      "audio pitch control element");
+
   gobject_class->set_property = gst_pitch_set_property;
   gobject_class->get_property = gst_pitch_get_property;
   gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_pitch_dispose);
@@ -184,6 +181,8 @@ gst_pitch_init (GstPitch * pitch, GstPitchClass * pitch_class)
       GST_DEBUG_FUNCPTR (gst_pad_proxy_getcaps));
   gst_element_add_pad (GST_ELEMENT (pitch), pitch->srcpad);
 
+  gst_element_no_more_pads (GST_ELEMENT (pitch));
+
   pitch->priv->st = new soundtouch::SoundTouch ();
 
   pitch->tempo = 1.0;
@@ -197,6 +196,7 @@ gst_pitch_init (GstPitch * pitch, GstPitchClass * pitch_class)
   pitch->priv->st->setPitch (pitch->pitch);
 
   pitch->priv->stream_time_ratio = 1.0;
+  pitch->min_latency = pitch->max_latency = 0;
 }
 
 
@@ -206,13 +206,23 @@ gst_pitch_dispose (GObject * object)
   GstPitch *pitch = GST_PITCH (object);
 
   if (pitch->priv->st) {
-    delete (pitch->priv->st);
+    delete pitch->priv->st;
+
     pitch->priv->st = NULL;
   }
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
+static void
+gst_pitch_update_duration (GstPitch * pitch)
+{
+  GstMessage *m;
+
+  m = gst_message_new_duration (GST_OBJECT (pitch), GST_FORMAT_TIME, -1);
+  gst_element_post_message (GST_ELEMENT (pitch), m);
+}
+
 static void
 gst_pitch_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -225,21 +235,26 @@ gst_pitch_set_property (GObject * object, guint prop_id,
       pitch->tempo = g_value_get_float (value);
       pitch->priv->stream_time_ratio = pitch->tempo * pitch->rate;
       pitch->priv->st->setTempo (pitch->tempo);
+      GST_OBJECT_UNLOCK (pitch);
+      gst_pitch_update_duration (pitch);
       break;
     case ARG_RATE:
       pitch->rate = g_value_get_float (value);
       pitch->priv->stream_time_ratio = pitch->tempo * pitch->rate;
       pitch->priv->st->setRate (pitch->rate);
+      GST_OBJECT_UNLOCK (pitch);
+      gst_pitch_update_duration (pitch);
       break;
     case ARG_PITCH:
       pitch->pitch = g_value_get_float (value);
       pitch->priv->st->setPitch (pitch->pitch);
+      GST_OBJECT_UNLOCK (pitch);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      GST_OBJECT_UNLOCK (pitch);
       break;
   }
-  GST_OBJECT_UNLOCK (pitch);
 }
 
 static void
@@ -301,7 +316,6 @@ gst_pitch_sink_setcaps (GstPad * pad, GstCaps * caps)
 
   /* calculate sample size */
   pitch->sample_size = (sizeof (gfloat) * channels);
-  pitch->sample_duration = gst_util_uint64_scale_int (GST_SECOND, 1, rate);
 
   GST_OBJECT_UNLOCK (pitch);
 
@@ -322,8 +336,6 @@ gst_pitch_forward_buffer (GstPitch * pitch, GstBuffer * buffer)
   pitch->next_buffer_offset += samples;
   GST_BUFFER_OFFSET_END (buffer) = pitch->next_buffer_offset;
 
-  gst_buffer_set_caps (buffer, GST_PAD_CAPS (pitch->srcpad));
-
   GST_LOG ("pushing buffer [%" GST_TIME_FORMAT "]-[%" GST_TIME_FORMAT
       "] (%d samples)", GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)),
       GST_TIME_ARGS (pitch->next_buffer_time), samples);
@@ -345,16 +357,25 @@ gst_pitch_prepare_buffer (GstPitch * pitch)
 
   samples = pitch->priv->st->numSamples ();
   if (samples == 0)
-    return NULL;;
+    return NULL;
+
+  if (gst_pad_alloc_buffer_and_set_caps (pitch->srcpad, GST_BUFFER_OFFSET_NONE,
+          samples * pitch->sample_size, GST_PAD_CAPS (pitch->srcpad), &buffer)
+      != GST_FLOW_OK) {
+    buffer = gst_buffer_new_and_alloc (samples * pitch->sample_size);
+    gst_buffer_set_caps (buffer, GST_PAD_CAPS (pitch->srcpad));
+  }
 
-  buffer = gst_buffer_new_and_alloc (samples * pitch->sample_size);
   samples =
       priv->st->receiveSamples ((gfloat *) GST_BUFFER_DATA (buffer), samples);
 
-  if (samples <= 0)
+  if (samples <= 0) {
+    gst_buffer_unref (buffer);
     return NULL;
+  }
 
-  GST_BUFFER_DURATION (buffer) = samples * pitch->sample_duration;
+  GST_BUFFER_DURATION (buffer) =
+      gst_util_uint64_scale (samples, GST_SECOND, pitch->samplerate);
   /* temporary store samples here, to avoid having to recalculate this */
   GST_BUFFER_OFFSET (buffer) = (gint64) samples;
 
@@ -365,7 +386,7 @@ gst_pitch_prepare_buffer (GstPitch * pitch)
  * samples are sent out here as strictly necessary, because soundtouch could
  * append zero samples, which could disturb looping.  */
 static GstFlowReturn
-gst_pitch_flush_buffer (GstPitch * pitch)
+gst_pitch_flush_buffer (GstPitch * pitch, gboolean send)
 {
   GstBuffer *buffer;
 
@@ -375,6 +396,9 @@ gst_pitch_flush_buffer (GstPitch * pitch)
     return GST_FLOW_OK;
 
   pitch->priv->st->flush ();
+  if (!send)
+    return GST_FLOW_OK;
+
   buffer = gst_pitch_prepare_buffer (pitch);
 
   if (!buffer)
@@ -410,15 +434,22 @@ gst_pitch_src_event (GstPad * pad, GstEvent * event)
       gst_event_parse_seek (event, &rate, &format, &flags,
           &cur_type, &cur, &stop_type, &stop);
 
-      cur = (gint64) (cur * stream_time_ratio);
-      stop = (gint64) (stop * stream_time_ratio);
-
       gst_event_unref (event);
 
-      event = gst_event_new_seek (rate, format, flags,
-          cur_type, cur, stop_type, stop);
+      if (format == GST_FORMAT_TIME || format == GST_FORMAT_DEFAULT) {
+        cur = (gint64) (cur * stream_time_ratio);
+        if (stop != -1)
+          stop = (gint64) (stop * stream_time_ratio);
+
+        event = gst_event_new_seek (rate, format, flags,
+            cur_type, cur, stop_type, stop);
+        res = gst_pad_event_default (pad, event);
+      } else {
+        GST_WARNING_OBJECT (pitch,
+            "Seeking only supported in TIME or DEFAULT format");
+        res = FALSE;
+      }
 
-      res = gst_pad_event_default (pad, event);
       break;
     }
     default:
@@ -439,30 +470,35 @@ gst_pitch_convert (GstPitch * pitch,
     GstFormat * dst_format, gint64 * dst_value)
 {
   gboolean res = TRUE;
-  GstClockTime sample_duration;
   guint sample_size;
+  gint samplerate;
 
   g_return_val_if_fail (dst_format && dst_value, FALSE);
 
   GST_OBJECT_LOCK (pitch);
-  sample_duration = pitch->sample_duration;
   sample_size = pitch->sample_size;
+  samplerate = pitch->samplerate;
   GST_OBJECT_UNLOCK (pitch);
 
-  if (sample_size == 0 || sample_duration == 0 ||
-      sample_duration == GST_CLOCK_TIME_NONE) {
+  if (sample_size == 0 || samplerate == 0) {
     return FALSE;
   }
 
+  if (src_format == *dst_format || src_value == -1) {
+    *dst_value = src_value;
+    return TRUE;
+  }
+
   switch (src_format) {
     case GST_FORMAT_BYTES:
       switch (*dst_format) {
         case GST_FORMAT_TIME:
-          *dst_value = src_value / sample_size;
-          *dst_value *= sample_duration;
+          *dst_value =
+              gst_util_uint64_scale_int (src_value, GST_SECOND,
+              sample_size * samplerate);
           break;
         case GST_FORMAT_DEFAULT:
-          *dst_value = src_value / sample_size;
+          *dst_value = gst_util_uint64_scale_int (src_value, 1, sample_size);
           break;
         default:
           res = FALSE;
@@ -472,11 +508,13 @@ gst_pitch_convert (GstPitch * pitch,
     case GST_FORMAT_TIME:
       switch (*dst_format) {
         case GST_FORMAT_BYTES:
-          *dst_value = src_value / sample_duration;
-          *dst_value *= sample_size;
+          *dst_value =
+              gst_util_uint64_scale_int (src_value, samplerate * sample_size,
+              GST_SECOND);
           break;
         case GST_FORMAT_DEFAULT:
-          *dst_value = src_value / sample_duration;
+          *dst_value =
+              gst_util_uint64_scale_int (src_value, samplerate, GST_SECOND);
           break;
         default:
           res = FALSE;
@@ -486,10 +524,11 @@ gst_pitch_convert (GstPitch * pitch,
     case GST_FORMAT_DEFAULT:
       switch (*dst_format) {
         case GST_FORMAT_BYTES:
-          *dst_value = src_value * sample_size;
+          *dst_value = gst_util_uint64_scale_int (src_value, sample_size, 1);
           break;
         case GST_FORMAT_TIME:
-          *dst_value = src_value * sample_duration;
+          *dst_value =
+              gst_util_uint64_scale_int (src_value, GST_SECOND, samplerate);
           break;
         default:
           res = FALSE;
@@ -511,6 +550,7 @@ gst_pitch_get_query_types (GstPad * pad)
     GST_QUERY_POSITION,
     GST_QUERY_DURATION,
     GST_QUERY_CONVERT,
+    GST_QUERY_LATENCY,
     GST_QUERY_NONE
   };
 
@@ -524,11 +564,13 @@ gst_pitch_src_query (GstPad * pad, GstQuery * query)
   gboolean res = FALSE;
   gfloat stream_time_ratio;
   gint64 next_buffer_offset;
+  GstClockTime next_buffer_time;
 
   pitch = GST_PITCH (gst_pad_get_parent (pad));
   GST_LOG ("%s query", GST_QUERY_TYPE_NAME (query));
   GST_OBJECT_LOCK (pitch);
   stream_time_ratio = pitch->priv->stream_time_ratio;
+  next_buffer_time = pitch->next_buffer_time;
   next_buffer_offset = pitch->next_buffer_offset;
   GST_OBJECT_UNLOCK (pitch);
 
@@ -566,9 +608,9 @@ gst_pitch_src_query (GstPad * pad, GstQuery * query)
         break;
       }
 
-      if (dst_format != GST_FORMAT_DEFAULT) {
-        res = gst_pitch_convert (pitch, GST_FORMAT_DEFAULT,
-            next_buffer_offset, &dst_format, &dst_value);
+      if (dst_format == GST_FORMAT_TIME) {
+        dst_value = next_buffer_time;
+        res = TRUE;
       } else {
         dst_value = next_buffer_offset;
         res = TRUE;
@@ -596,6 +638,46 @@ gst_pitch_src_query (GstPad * pad, GstQuery * query)
       }
       break;
     }
+    case GST_QUERY_LATENCY:
+    {
+      GstClockTime min, max;
+      gboolean live;
+      GstPad *peer;
+
+      if ((peer = gst_pad_get_peer (pitch->sinkpad))) {
+        if ((res = gst_pad_query (peer, query))) {
+          gst_query_parse_latency (query, &live, &min, &max);
+
+          GST_DEBUG ("Peer latency: min %"
+              GST_TIME_FORMAT " max %" GST_TIME_FORMAT,
+              GST_TIME_ARGS (min), GST_TIME_ARGS (max));
+
+          /* add our own latency */
+
+          GST_DEBUG ("Our latency: min %" GST_TIME_FORMAT
+              ", max %" GST_TIME_FORMAT,
+              GST_TIME_ARGS (pitch->min_latency),
+              GST_TIME_ARGS (pitch->max_latency));
+
+          min += pitch->min_latency;
+          if (max != GST_CLOCK_TIME_NONE)
+            max += pitch->max_latency;
+          else
+            max = pitch->max_latency;
+
+          GST_DEBUG ("Calculated total latency : min %"
+              GST_TIME_FORMAT " max %" GST_TIME_FORMAT,
+              GST_TIME_ARGS (min), GST_TIME_ARGS (max));
+          g_print ("Calculated total latency : min %"
+              GST_TIME_FORMAT " max %" GST_TIME_FORMAT,
+              GST_TIME_ARGS (min), GST_TIME_ARGS (max));
+
+          gst_query_set_latency (query, live, min, max);
+        }
+        gst_object_unref (peer);
+      }
+      break;
+    }
     default:
       res = gst_pad_query_default (pad, query);
       break;
@@ -605,6 +687,80 @@ gst_pitch_src_query (GstPad * pad, GstQuery * query)
   return res;
 }
 
+/* this function returns FALSE if not enough data is known to transform the
+ * segment into proper downstream values.  If the function does return false
+ * the sgement should be stalled until enough information is available.
+ * If the funtion returns TRUE, event will be replaced by the new downstream
+ * compatible event.
+ */
+static gboolean
+gst_pitch_process_segment (GstPitch * pitch, GstEvent ** event)
+{
+  GstFormat format, conv_format;
+  gint64 start_value, stop_value, base;
+  gint64 next_offset = 0, next_time = 0;
+  gboolean update = FALSE;
+  gdouble rate;
+  gfloat stream_time_ratio;
+
+  g_return_val_if_fail (event, FALSE);
+
+  GST_OBJECT_LOCK (pitch);
+  stream_time_ratio = pitch->priv->stream_time_ratio;
+  GST_OBJECT_UNLOCK (pitch);
+
+  gst_event_parse_new_segment (*event, &update, &rate, &format, &start_value,
+      &stop_value, &base);
+
+  if (format != GST_FORMAT_TIME && format != GST_FORMAT_DEFAULT) {
+    GST_WARNING_OBJECT (pitch,
+        "Only NEWSEGMENT in TIME or DEFAULT format supported, sending"
+        "open ended NEWSEGMENT in TIME format.");
+    gst_event_unref (*event);
+    *event =
+        gst_event_new_new_segment (update, rate, GST_FORMAT_TIME, 0, -1, 0);
+    start_value = 0;
+    stop_value = -1;
+    base = 0;
+  }
+
+  GST_LOG_OBJECT (pitch->sinkpad, "segment %lld - %lld (%d)", start_value,
+      stop_value, format);
+
+  if (stream_time_ratio == 0) {
+    GST_LOG_OBJECT (pitch->sinkpad, "stream_time_ratio is zero");
+    return FALSE;
+  }
+
+  start_value = (gint64) (start_value / stream_time_ratio);
+  if (stop_value != -1)
+    stop_value = (gint64) (stop_value / stream_time_ratio);
+  base = (gint64) (base / stream_time_ratio);
+
+  conv_format = GST_FORMAT_TIME;
+  if (!gst_pitch_convert (pitch, format, start_value, &conv_format, &next_time)) {
+    GST_LOG_OBJECT (pitch->sinkpad,
+        "could not convert segment start value to time");
+    return FALSE;
+  }
+
+  conv_format = GST_FORMAT_DEFAULT;
+  if (!gst_pitch_convert (pitch, format, start_value, &conv_format,
+          &next_offset)) {
+    GST_LOG_OBJECT (pitch->sinkpad,
+        "could not convert segment start value to offset");
+    return FALSE;
+  }
+
+  pitch->next_buffer_time = next_time;
+  pitch->next_buffer_offset = next_offset;
+
+  gst_event_unref (*event);
+  *event = gst_event_new_new_segment (update, rate, format, start_value,
+      stop_value, base);
+
+  return TRUE;
+}
 
 static gboolean
 gst_pitch_sink_event (GstPad * pad, GstEvent * event)
@@ -617,46 +773,117 @@ gst_pitch_sink_event (GstPad * pad, GstEvent * event)
   GST_LOG_OBJECT (pad, "received %s event", GST_EVENT_TYPE_NAME (event));
 
   switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_NEWSEGMENT:
+    case GST_EVENT_FLUSH_STOP:
+      gst_pitch_flush_buffer (pitch, FALSE);
+      pitch->priv->st->clear ();
+      pitch->min_latency = pitch->max_latency = 0;
+      break;
     case GST_EVENT_EOS:
-      gst_pitch_flush_buffer (pitch);
+      gst_pitch_flush_buffer (pitch, TRUE);
+      pitch->priv->st->clear ();
+      pitch->min_latency = pitch->max_latency = 0;
+      break;
+    case GST_EVENT_NEWSEGMENT:
+      if (!gst_pitch_process_segment (pitch, &event)) {
+        GST_LOG_OBJECT (pad, "not enough data known, stalling segment");
+        if (GST_PITCH_GET_PRIVATE (pitch)->pending_segment)
+          gst_event_unref (GST_PITCH_GET_PRIVATE (pitch)->pending_segment);
+        GST_PITCH_GET_PRIVATE (pitch)->pending_segment = event;
+        event = NULL;
+      }
+      pitch->priv->st->clear ();
+      pitch->min_latency = pitch->max_latency = 0;
       break;
     default:
       break;
   }
 
   /* and forward it */
-  res = gst_pad_event_default (pad, event);
+  if (event)
+    res = gst_pad_event_default (pad, event);
 
   gst_object_unref (pitch);
   return res;
 }
 
+static void
+gst_pitch_update_latency (GstPitch * pitch, GstClockTime timestamp)
+{
+  GstClockTimeDiff current_latency, min_latency, max_latency;
+
+  current_latency =
+      (GstClockTimeDiff) (timestamp / pitch->priv->stream_time_ratio) -
+      pitch->next_buffer_time;
+
+  min_latency = MIN (pitch->min_latency, current_latency);
+  max_latency = MAX (pitch->max_latency, current_latency);
+
+  if (pitch->min_latency != min_latency || pitch->max_latency != max_latency) {
+    pitch->min_latency = min_latency;
+    pitch->max_latency = max_latency;
+
+    /* FIXME: what about the LATENCY event? It only has
+     * one latency value, should it be current, min or max?
+     * Should it include upstream latencies?
+     */
+
+    gst_element_post_message (GST_ELEMENT (pitch),
+        gst_message_new_latency (GST_OBJECT (pitch)));
+  }
+}
+
 static GstFlowReturn
 gst_pitch_chain (GstPad * pad, GstBuffer * buffer)
 {
   GstPitch *pitch;
   GstPitchPrivate *priv;
+  GstClockTime timestamp;
 
   pitch = GST_PITCH (GST_PAD_PARENT (pad));
   priv = GST_PITCH_GET_PRIVATE (pitch);
 
   gst_object_sync_values (G_OBJECT (pitch), pitch->next_buffer_time);
 
+  timestamp = GST_BUFFER_TIMESTAMP (buffer);
+
   /* push the received samples on the soundtouch buffer */
   GST_LOG_OBJECT (pitch, "incoming buffer (%d samples)",
       (gint) (GST_BUFFER_SIZE (buffer) / pitch->sample_size));
 
+  if (GST_PITCH_GET_PRIVATE (pitch)->pending_segment) {
+    GstEvent *event =
+        gst_event_copy (GST_PITCH_GET_PRIVATE (pitch)->pending_segment);
+
+    GST_LOG_OBJECT (pitch, "processing stalled segment");
+    if (!gst_pitch_process_segment (pitch, &event)) {
+      gst_event_unref (event);
+      return GST_FLOW_ERROR;
+    }
+
+    if (!gst_pad_event_default (pitch->sinkpad, event)) {
+      gst_event_unref (event);
+      return GST_FLOW_ERROR;
+    }
+
+    gst_event_unref (GST_PITCH_GET_PRIVATE (pitch)->pending_segment);
+    GST_PITCH_GET_PRIVATE (pitch)->pending_segment = NULL;
+  }
+
   priv->st->putSamples ((gfloat *) GST_BUFFER_DATA (buffer),
       GST_BUFFER_SIZE (buffer) / pitch->sample_size);
   gst_buffer_unref (buffer);
 
+  /* Calculate latency */
+
+  gst_pitch_update_latency (pitch, timestamp);
+
   /* and try to extract some samples from the soundtouch buffer */
   if (!priv->st->isEmpty ()) {
     GstBuffer *out_buffer;
 
     out_buffer = gst_pitch_prepare_buffer (pitch);
-    return gst_pitch_forward_buffer (pitch, out_buffer);
+    if (out_buffer)
+      return gst_pitch_forward_buffer (pitch, out_buffer);
   }
 
   return GST_FLOW_OK;
@@ -674,6 +901,8 @@ gst_pitch_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       pitch->next_buffer_time = 0;
       pitch->next_buffer_offset = 0;
+      pitch->priv->st->clear ();
+      pitch->min_latency = pitch->max_latency = 0;
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
@@ -687,7 +916,13 @@ gst_pitch_change_state (GstElement * element, GstStateChange transition)
 
   switch (transition) {
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
+      break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
+      if (GST_PITCH_GET_PRIVATE (pitch)->pending_segment) {
+        gst_event_unref (GST_PITCH_GET_PRIVATE (pitch)->pending_segment);
+        GST_PITCH_GET_PRIVATE (pitch)->pending_segment = NULL;
+      }
+      break;
     case GST_STATE_CHANGE_READY_TO_NULL:
     default:
       break;
@@ -695,20 +930,3 @@ gst_pitch_change_state (GstElement * element, GstStateChange transition)
 
   return ret;
 }
-
-static gboolean
-plugin_init (GstPlugin * plugin)
-{
-  gst_controller_init (NULL, NULL);
-
-  GST_DEBUG_CATEGORY_INIT (pitch_debug, "pitch", 0,
-      "audio pitch control element");
-
-  return gst_element_register (plugin, "pitch", GST_RANK_NONE, GST_TYPE_PITCH);
-}
-
-GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
-    GST_VERSION_MINOR,
-    "soundtouch",
-    "Audio Pitch Controller",
-    plugin_init, VERSION, "LGPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
