@@ -23,18 +23,14 @@
  * SECTION:element-timidity
  * @see_also: wildmidi
  *
- * <refsect2>
- * <para>
  * This element renders midi-files as audio streams using
  * <ulink url="http://timidity.sourceforge.net/">Timidity</ulink>.
- * </para>
- * <para>
- * </para>
+ * 
+ * <refsect2>
  * <title>Example pipeline</title>
- * <programlisting>
+ * |[
  * gst-launch filesrc location=song.mid ! timidity ! alsasink
- * </programlisting>
- * This example pipeline will parse the midi and render to raw audio which is
+ * ]| This example pipeline will parse the midi and render to raw audio which is
  * played via alsa.
  * </refsect2>
  */
@@ -681,8 +677,6 @@ gst_timidity_loop (GstPad * sinkpad)
   }
 
   if (timidity->o_segment_changed) {
-    GST_DEBUG_OBJECT (timidity, "segment changed");
-
     GstSegment *segment = gst_timidity_get_segment (timidity, GST_FORMAT_TIME,
         !timidity->o_new_segment);
 
