@@ -25,18 +25,17 @@
      Olivier Lapicque <olivierl@jps.net>  
 */
 
-/*
+/**
  * SECTION:element-modplug
  * 
- * <refsect2>
- * <para>
  * Modplug uses the <ulink url="http://modplug-xmms.sourceforge.net/">modplug</ulink>
  * library to decode tracked music in the MOD/S3M/XM/IT and related formats.
- * </para>
+ * 
+ * <refsect2>
  * <title>Example pipeline</title>
- * <programlisting>
+ * |[
  * gst-launch -v filesrc location=1990s-nostalgia.xm ! modplug ! audioconvert ! alsasink
- * </programlisting>
+ * ]| Play a FastTracker xm file.
  * </refsect2>
  */
 
@@ -56,14 +55,6 @@
 GST_DEBUG_CATEGORY_STATIC (modplug_debug);
 #define GST_CAT_DEFAULT modplug_debug
 
-/* elementfactory information */
-GstElementDetails modplug_details = {
-  "ModPlug",
-  "Codec/Decoder/Audio",
-  "Module decoder based on modplug engine",
-  "Jeremy SIMON <jsimon13@yahoo.fr>"
-};
-
 enum
 {
   ARG_0,
@@ -93,33 +84,31 @@ enum
 #define DEFAULT_OVERSAMP         TRUE
 #define DEFAULT_NOISE_REDUCTION  TRUE
 
-#define SRC_CAPS \
-  "audio/x-raw-int,"                              \
-  " endianness = (int) BYTE_ORDER,"               \
-  " signed = (boolean) true,"                     \
-  " width = (int) 16,"                            \
-  " depth = (int) 16,"                            \
-  " rate = (int) { 8000, 11025, 22050, 44100 },"  \
-  " channels = (int) 2; "                         \
-  "audio/x-raw-int,"                              \
-  " endianness = (int) BYTE_ORDER,"               \
-  " signed = (boolean) false,"                    \
-  " width = (int) 8,"                             \
-  " depth = (int) 8,"                             \
-  " rate = (int) { 8000, 11025, 22050, 44100 }, " \
-  " channels = (int) [ 1, 2 ]"
-
 static GstStaticPadTemplate modplug_src_template_factory =
-GST_STATIC_PAD_TEMPLATE ("src",
+    GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (SRC_CAPS));
+    GST_STATIC_CAPS ("audio/x-raw-int,"
+        " endianness = (int) BYTE_ORDER,"
+        " signed = (boolean) true,"
+        " width = (int) 16,"
+        " depth = (int) 16,"
+        " rate = (int) { 8000, 11025, 22050, 44100 },"
+        " channels = (int) 2; "
+        "audio/x-raw-int,"
+        " endianness = (int) BYTE_ORDER,"
+        " signed = (boolean) false,"
+        " width = (int) 8,"
+        " depth = (int) 8,"
+        " rate = (int) { 8000, 11025, 22050, 44100 }, "
+        " channels = (int) [ 1, 2 ]"));
 
 static GstStaticPadTemplate modplug_sink_template_factory =
-GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("audio/x-mod"));
+    GST_STATIC_CAPS ("audio/x-mod; audio/x-xm; audio/x-it; audio/x-s3m; "
+        "audio/x-stm"));
 
 static void gst_modplug_dispose (GObject * object);
 static void gst_modplug_set_property (GObject * object,
@@ -150,7 +139,10 @@ gst_modplug_base_init (gpointer g_class)
       gst_static_pad_template_get (&modplug_sink_template_factory));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&modplug_src_template_factory));
-  gst_element_class_set_details (element_class, &modplug_details);
+
+  gst_element_class_set_details_simple (element_class, "ModPlug",
+      "Codec/Decoder/Audio", "Module decoder based on modplug engine",
+      "Jeremy SIMON <jsimon13@yahoo.fr>");
 
   GST_DEBUG_CATEGORY_INIT (modplug_debug, "modplug", 0, "ModPlug element");
 }
@@ -217,7 +209,7 @@ gst_modplug_class_init (GstModPlugClass * klass)
           "noise reduction", DEFAULT_NOISE_REDUCTION,
           (GParamFlags) G_PARAM_READWRITE));
 
-  gstelement_class->change_state = GST_DEBUG_FUNCPTR (gst_modplug_change_state);
+  gstelement_class->change_state = gst_modplug_change_state;
 }
 
 static void
@@ -458,15 +450,18 @@ gst_modplug_load_song (GstModPlug * modplug)
   GstStructure *structure;
   gint depth;
 
-  GST_DEBUG_OBJECT (modplug, "Loading song");
-
-  modplug->mSoundFile = new CSoundFile;
+  GST_DEBUG_OBJECT (modplug, "Setting caps");
 
   /* negotiate srcpad caps */
-  othercaps = gst_pad_get_allowed_caps (modplug->srcpad);
-  structure = gst_caps_get_structure (othercaps, 0);
-  newcaps = gst_caps_copy_nth (othercaps, 0);
-  gst_caps_unref (othercaps);
+  if ((othercaps = gst_pad_get_allowed_caps (modplug->srcpad))) {
+    newcaps = gst_caps_copy_nth (othercaps, 0);
+    gst_caps_unref (othercaps);
+  } else {
+    GST_WARNING ("no allowed caps on srcpad, no peer linked");
+    /* FIXME: this can be done in a better way */
+    newcaps =
+        gst_caps_copy_nth (gst_pad_get_pad_template_caps (modplug->srcpad), 0);
+  }
   gst_pad_fixate_caps (modplug->srcpad, newcaps);
   gst_pad_set_caps (modplug->srcpad, newcaps);
 
@@ -480,6 +475,10 @@ gst_modplug_load_song (GstModPlug * modplug)
   modplug->read_samples = 1152;
   modplug->read_bytes = modplug->read_samples * modplug->channel * depth / 8;
 
+  GST_DEBUG_OBJECT (modplug, "Loading song");
+
+  modplug->mSoundFile = new CSoundFile;
+
   if (modplug->_16bit)
     modplug->mSoundFile->SetWaveConfig (modplug->frequency, 16,
         modplug->channel);
