@@ -79,9 +79,9 @@ static gboolean gst_dshowaudiodec_flush (GstDshowAudioDec * adec);
 static gboolean gst_dshowaudiodec_get_filter_settings (GstDshowAudioDec * adec);
 static gboolean gst_dshowaudiodec_setup_graph (GstDshowAudioDec * adec);
 
-/* gobal variable */
-const long bitrates[2][3][16] = {
-  /* version 0 */
+/* global variable */
+static const long bitrates[2][3][16] = {
+  /* mpeg 1 */
   {
         /* one list per layer 1-3 */
         {0, 32000, 48000, 56000, 64000, 80000, 96000, 112000, 128000, 144000,
@@ -91,7 +91,7 @@ const long bitrates[2][3][16] = {
         {0, 8000, 16000, 24000, 32000, 40000, 48000, 56000, 64000, 80000, 96000,
             112000, 128000, 144000, 160000, 0},
       },
-  /* version 1 */
+  /* mpeg 2 */
   {
         /* one list per layer 1-3 */
         {0, 32000, 64000, 96000, 128000, 160000, 192000, 224000, 256000,
@@ -193,9 +193,9 @@ static const CodecEntry audio_dec_codecs[] = {
 };
 
 /* Private map used when dshowadec_mpeg is loaded with layer=1 or 2.
- * The problem is that gstreamer don't care about caps like layer when connecting pads.
+ * The problem is that gstreamer doesn't care about caps like layer when connecting pads.
  * So I've only one element handling mpeg audio in the public codecs map and 
- * when it's loaded for mp3, I'm releasing mpeg audio decoder and replace it by 
+ * when it's loaded for mp3, I release the mpeg audio decoder and replace it by 
  * the one described in this private map.
 */
 static const CodecEntry audio_mpeg_1_2[] = { "dshowadec_mpeg_1_2",
@@ -308,6 +308,8 @@ gst_dshowaudiodec_init (GstDshowAudioDec * adec,
   adec->layer = 0;
   adec->codec_data = NULL;
 
+  adec->last_ret = GST_FLOW_OK;
+
   CoInitializeEx (NULL, COINIT_MULTITHREADED);
 }
 
@@ -420,7 +422,6 @@ end:
 static GstFlowReturn
 gst_dshowaudiodec_chain (GstPad * pad, GstBuffer * buffer)
 {
-  GstFlowReturn ret = GST_FLOW_OK;
   GstDshowAudioDec *adec = (GstDshowAudioDec *) gst_pad_get_parent (pad);
   gboolean discount = FALSE;
 
@@ -436,13 +437,20 @@ gst_dshowaudiodec_chain (GstPad * pad, GstBuffer * buffer)
 
     /* setup dshow graph */
     if (!gst_dshowaudiodec_setup_graph (adec)) {
-      return GST_FLOW_ERROR;
+      adec->last_ret = GST_FLOW_ERROR;
+      goto beach;
     }
   }
 
   if (!adec->gstdshowsrcfilter) {
     /* we are not setup */
-    ret = GST_FLOW_WRONG_STATE;
+    adec->last_ret = GST_FLOW_WRONG_STATE;
+    goto beach;
+  }
+
+  if (GST_FLOW_IS_FATAL (adec->last_ret)) {
+    GST_DEBUG_OBJECT (adec, "last decoding iteration generated a fatal error "
+        "%s", gst_flow_get_name (adec->last_ret));
     goto beach;
   }
 
@@ -470,7 +478,7 @@ gst_dshowaudiodec_chain (GstPad * pad, GstBuffer * buffer)
 beach:
   gst_buffer_unref (buffer);
   gst_object_unref (adec);
-  return ret;
+  return adec->last_ret;
 }
 
 static gboolean
@@ -509,7 +517,7 @@ gst_dshowaudiodec_push_buffer (byte * buffer, long size, byte * src_object,
   /* buffer is in our segment allocate a new out buffer and clip it if needed */
 
   /* allocate a new buffer for raw audio */
-  gst_pad_alloc_buffer (adec->srcpad, GST_BUFFER_OFFSET_NONE,
+  adec->last_ret = gst_pad_alloc_buffer (adec->srcpad, GST_BUFFER_OFFSET_NONE,
       size, GST_PAD_CAPS (adec->srcpad), &out_buf);
   if (!out_buf) {
     GST_CAT_ERROR_OBJECT (dshowaudiodec_debug, adec,
@@ -518,10 +526,10 @@ gst_dshowaudiodec_push_buffer (byte * buffer, long size, byte * src_object,
   }
 
   /* set buffer properties */
-  GST_BUFFER_SIZE (out_buf) = size;
   GST_BUFFER_TIMESTAMP (out_buf) = buf_start;
   GST_BUFFER_DURATION (out_buf) = buf_stop - buf_start;
-  memcpy (GST_BUFFER_DATA (out_buf), buffer, size);
+  memcpy (GST_BUFFER_DATA (out_buf), buffer,
+      MIN (size, GST_BUFFER_SIZE (out_buf)));
 
   /* we have to remove some heading samples */
   if (clip_start > buf_start) {
@@ -560,7 +568,7 @@ gst_dshowaudiodec_push_buffer (byte * buffer, long size, byte * src_object,
           GST_BUFFER_DURATION (out_buf)),
       GST_TIME_ARGS (GST_BUFFER_DURATION (out_buf)));
 
-  gst_pad_push (adec->srcpad, out_buf);
+  adec->last_ret = gst_pad_push (adec->srcpad, out_buf);
 
   return TRUE;
 }
@@ -608,6 +616,9 @@ gst_dshowaudiodec_sink_event (GstPad * pad, GstEvent * event)
       ret = gst_pad_event_default (pad, event);
       break;
   }
+
+  gst_object_unref (adec);
+
   return ret;
 }
 
@@ -652,10 +663,10 @@ gst_dshowaudiodec_setup_graph (GstDshowAudioDec * adec)
           codec_entry->input_subtype,
           codec_entry->output_majortype,
           codec_entry->output_subtype,
-          codec_entry->prefered_filter_substring, &adec->decfilter);
+          codec_entry->preferred_filter_substring, &adec->decfilter);
       IFilterGraph_AddFilter (adec->filtergraph, adec->decfilter, L"decoder");
     } else {
-      /* mp3 don't need to negociate with MPEG1WAVEFORMAT */
+      /* mp3 doesn't need to negotiate with MPEG1WAVEFORMAT */
       adec->layer = 0;
     }
   }
@@ -899,7 +910,6 @@ gst_dshowaudiodec_setup_graph (GstDshowAudioDec * adec)
   ret = TRUE;
   adec->setup = TRUE;
 end:
-  gst_object_unref (adec);
   if (input_format)
     g_free (input_format);
   if (gstdshowinterface)
@@ -1007,7 +1017,7 @@ gst_dshowaudiodec_create_graph_and_filters (GstDshowAudioDec * adec)
           klass->entry->input_subtype,
           klass->entry->output_majortype,
           klass->entry->output_subtype,
-          klass->entry->prefered_filter_substring, &adec->decfilter)) {
+          klass->entry->preferred_filter_substring, &adec->decfilter)) {
     GST_ELEMENT_ERROR (adec, STREAM, FAILED,
         ("Can't create an instance of the decoder filter"), (NULL));
     goto error;
@@ -1143,7 +1153,7 @@ dshow_adec_register (GstPlugin * plugin)
             audio_dec_codecs[i].input_subtype,
             audio_dec_codecs[i].output_majortype,
             audio_dec_codecs[i].output_subtype,
-            audio_dec_codecs[i].prefered_filter_substring, NULL)) {
+            audio_dec_codecs[i].preferred_filter_substring, NULL)) {
 
       GST_CAT_DEBUG (dshowaudiodec_debug, "Registering %s",
           audio_dec_codecs[i].element_name);
