@@ -19,22 +19,17 @@
 
 /**
  * SECTION:element-oss4src
- * @short_description: record sound from your sound card using OSS4
  *
- * <refsect2>
- * <para>
  * This element lets you record sound using the Open Sound System (OSS)
  * version 4.
- * </para>
+ * 
+ * <refsect2>
  * <title>Example pipelines</title>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch -v oss4src ! queue ! audioconvert ! vorbisenc ! oggmux ! filesink location=mymusic.ogg
- * </programlisting>
- * will record sound from your sound card using OSS4 and encode it to an
+ * ]| will record sound from your sound card using OSS4 and encode it to an
  * Ogg/Vorbis file (this will only work if your mixer settings are right
  * and the right inputs areenabled etc.)
- * </para>
  * </refsect2>
  *
  * Since: 0.10.7
@@ -256,11 +251,17 @@ gst_oss4_source_get_property (GObject * object, guint prop_id,
           g_value_set_string (value, oss->device_name);
           gst_oss4_source_close (GST_AUDIO_SRC (oss));
         } else {
-          g_value_set_string (value, NULL);
+          gchar *name = NULL;
+
+          gst_oss4_property_probe_find_device_name_nofd (GST_OBJECT (oss),
+              oss->device, &name);
+          g_value_set_string (value, name);
+          g_free (name);
         }
       } else {
         g_value_set_string (value, oss->device_name);
       }
+
       GST_OBJECT_UNLOCK (oss);
       break;
     default:
